#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Style properties default values filter"""

import logging
from typing import Dict, Type, Any

from ttconv.filters.isd_filter import ISDFilter
from ttconv.isd import ISD
from ttconv.model import ContentElement
from ttconv.style_properties import StyleProperty

LOGGER = logging.getLogger(__name__)


class DefaultStylePropertyValuesISDFilter(ISDFilter):
  """Filter that remove default style properties"""

  def __init__(self, style_property_default_values: Dict[Type[StyleProperty], Any]):
    self.style_property_default_values = style_property_default_values

  def _process_element(self, element: ContentElement):
    """Filter ISD element style properties"""

    element_styles = list(element.iter_styles())
    for style_prop in element_styles:

      value = element.get_style(style_prop)
      default_value = self.style_property_default_values.get(style_prop)

      parent = element.parent()
      if parent is not None and style_prop.is_inherited:
        # If the parent style property value has not been removed, it means
        # the value is not set to default, and so that the child style property
        # value may have been "forced" to the default value, so let's skip it.
        parent_value = parent.get_style(style_prop)
        if parent_value is not None and parent_value is not value:
          continue

      # Remove the style property if its value is default (and if it is not inherited)
      if default_value is not None and value == default_value:
        element.set_style(style_prop, None)

    for child in element:
      self._process_element(child)

  def process(self, isd: ISD):
    """Filter ISD document style properties"""
    LOGGER.debug("Apply default style properties filter to ISD.")

    for region in isd.iter_regions():
      self._process_element(region)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''Unit tests for the STL reader'''

# pylint: disable=R0201,C0115,C0116

import unittest
from ttconv.imsc.designators import IMSC_11_TEXT_PROFILE_DESIGNATOR
from ttconv.model import Br, Span
import ttconv.stl.reader
import ttconv.style_properties as styles
from ttconv.time_code import SmpteTimeCode, FPS_25

class STLReaderTests(unittest.TestCase):

  def test_irt_requirement_0056_001_modfied(self):
    '''Testing subtitles in different subtitle groups (SGN)'''
    with open("src/test/resources/stl/irt/requirement-0056-001_modified.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      
      div_list = list(doc.get_body())
      self.assertEqual(len(div_list), 3)
      
      p1_span = div_list[0].first_child().first_child()
      p2_span = div_list[0].first_child().next_sibling().first_child()
      p3_span = div_list[1].first_child().first_child()
      p4_span = div_list[2].first_child().first_child()
      self.assertEqual(p1_span.first_child().get_text(), "Subtitle 1 Group 1")
      self.assertEqual(p2_span.first_child().get_text(), "Subtitle 2 Group 1")
      self.assertEqual(p3_span.first_child().get_text(), "Subtitle 3 Group 2")
      self.assertEqual(p4_span.first_child().get_text(), "Subtitle 4 Group 3")

  def test_irt_requirement_0061_001(self):
    '''Testing Time Code In with value media 25 frames lowerbounds'''  
    with open("src/test/resources/stl/irt/requirement-0061-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      self.assertEqual(p.get_begin(),
                      SmpteTimeCode.parse("00:00:00:00",FPS_25).to_temporal_offset())

  def test_irt_requirement_0061_004_modified(self):
    '''Testing Time Code In with value media 25 frames upperbounds'''
    with open("src/test/resources/stl/irt/requirement-0061-004_modified.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      # both TCI and TCO are set to 23:59:59:24, so we expect the subtitle to be omitted and thus the body to be empty
      self.assertIsNone(doc.get_body().first_child())

  def test_irt_requirement_0062_001(self):
    '''Testing Time Code Out with value media 25 frames lower bound'''
    with open("src/test/resources/stl/irt/requirement-0062-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      # both TCI and TCO are set to 00:00:00:00, so we expect the subtitle to be omitted and thus the body to be empty
      self.assertIsNone(doc.get_body().first_child())

  def test_irt_requirement_0062_002_modified(self):
    '''Testing Time Code Out with value media 25 frames upper bound'''
    with open("src/test/resources/stl/irt/requirement-0062-002_modified.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      self.assertEqual(p.get_end(),
                       SmpteTimeCode.parse("23:59:59:24",FPS_25).to_temporal_offset())

  def test_irt_requirement_0067_001(self):
    '''Justification Code 01h mapping'''
    with open("src/test/resources/stl/irt/requirement-0067-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      self.assertEqual(p.get_style(styles.StyleProperties.TextAlign),
                       styles.TextAlignType.start)

  def test_irt_requirement_0068_001(self):
    '''Testing JC with value "02h"'''
    with open("src/test/resources/stl/irt/requirement-0068-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      self.assertEqual(p.get_style(styles.StyleProperties.TextAlign),
                       styles.TextAlignType.center)

  def test_irt_requirement_0069_001(self):
    '''Justification Code 03h mapping'''
    with open("src/test/resources/stl/irt/requirement-0069-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      self.assertEqual(p.get_style(styles.StyleProperties.TextAlign),
                       styles.TextAlignType.end)
  
  def test_irt_requirement_0071_002(self):
    '''Testing Text Field conversion to one or more tt:span elements (no StartBox/EndBox)'''
    with open("src/test/resources/stl/irt/requirement-0071-002.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      text = doc.get_body()\
                .first_child()\
                .first_child()\
                .first_child()\
                .first_child()\
                .get_text()
      self.assertEqual(text, "Test1 Test2") 

  def test_irt_requirement_0074_001(self):
    '''Testing that the newline element is mapped the tt:br element'''
    with open("src/test/resources/stl/irt/requirement-0074-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p_childs = list(doc.get_body().first_child().first_child())
      self.assertTrue(isinstance(p_childs[1], Br))
      
  
  def test_irt_requirement_0076_001(self):
    '''Testing Control Codes mapping for AlphaBlack'''
    with open("src/test/resources/stl/irt/requirement-0076-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)

  def test_irt_requirement_0076_002(self):
    '''Testing Control Codes mapping for AlphaWhite'''
    with open("src/test/resources/stl/irt/requirement-0076-002.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.white.value)

  def test_irt_requirement_0076_003(self):
    '''Testing Control Codes mapping for AlphaRed'''
    with open("src/test/resources/stl/irt/requirement-0076-003.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.red.value)

  def test_irt_requirement_0076_004(self):
    '''Testing Control Codes mapping for AlphaGreen'''
    with open("src/test/resources/stl/irt/requirement-0076-004.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.lime.value)
 
  def test_irt_requirement_0076_005(self):
    '''Testing Control Codes mapping for AlphaYellow,'''
    with open("src/test/resources/stl/irt/requirement-0076-005.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.yellow.value)

  def test_irt_requirement_0076_006(self):
    '''Testing Control Codes mapping for AlphaBlue'''
    with open("src/test/resources/stl/irt/requirement-0076-006.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.blue.value)
 
  def test_irt_requirement_0076_007(self):
    '''Testing Control Codes mapping for AlphaMagenta'''
    with open("src/test/resources/stl/irt/requirement-0076-007.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.magenta.value)
  
  def test_irt_requirement_0076_008(self):
    '''Testing Control Codes mapping for AlphaCyan'''
    with open("src/test/resources/stl/irt/requirement-0076-008.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(color_second_span, styles.NamedColors.cyan.value)
   
  def test_irt_requirement_0076_009(self):
    '''Testing Control Codes mapping for BlackBackground'''
    with open("src/test/resources/stl/irt/requirement-0076-009.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_first_span = spans[0].get_style(styles.StyleProperties.BackgroundColor)
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      self.assertEqual(background_color_first_span, styles.NamedColors.white.value)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
 
  def test_irt_requirement_0077_001(self):
    '''Testing DSC with value "1" and no style information'''
    with open("src/test/resources/stl/irt/requirement-0077-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_first_span = spans[0].get_style(styles.StyleProperties.BackgroundColor)
      self.assertEqual(background_color_first_span, styles.NamedColors.black.value)
 
  def test_irt_requirement_0077_002(self):
    '''Testing DSC with value "2" and no style information'''
    with open("src/test/resources/stl/irt/requirement-0077-002.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_first_span = spans[0].get_style(styles.StyleProperties.BackgroundColor)
      self.assertEqual(background_color_first_span, styles.NamedColors.black.value)

  def test_irt_requirement_0086_001(self):
    '''Testing EndBox element mapping with three closing EndBox 
       elements and without changing the colors'''
    with open("src/test/resources/stl/irt/requirement-0086-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      spans = [s for s in list(p) if isinstance(s, Span)]
      self.assertEqual(len(spans), 3)

  def test_irt_requirement_0087_001(self):
    '''Testing StartBox element mapping with 3x2 StartBox elements 
      and with referencing a style with the appropriate background 
      and foreground color'''
    with open("src/test/resources/stl/irt/requirement-0087-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      spans = [s for s in list(p) if isinstance(s, Span)]
      background_color_first_span = spans[0].get_style(styles.StyleProperties.BackgroundColor)
      color_first_span = spans[0].get_style(styles.StyleProperties.Color)
      self.assertEqual(len(spans), 3)
      self.assertEqual(background_color_first_span, styles.NamedColors.black.value)
      self.assertEqual(color_first_span, styles.NamedColors.white.value)   

  def test_irt_requirement_0090_001(self):
    '''Testing BlackBackground with AlphaBlack'''
    with open("src/test/resources/stl/irt/requirement-0090-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)

 
  def test_irt_requirement_0090_002(self):
    '''Testing BlackBackground with AlphaBlue'''
    with open("src/test/resources/stl/irt/requirement-0090-002.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.blue.value)
  
  def test_irt_requirement_0090_003(self):
    '''Testing BlackBackground with AlphaCyan'''
    with open("src/test/resources/stl/irt/requirement-0090-003.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.cyan.value)
  
  def test_irt_requirement_0090_004(self):
    '''Testing BlackBackground with AlphaGreen'''
    with open("src/test/resources/stl/irt/requirement-0090-004.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.lime.value)
  
  def test_irt_requirement_0090_005(self):
    '''Testing BlackBackground with AlphaMagenta'''
    with open("src/test/resources/stl/irt/requirement-0090-005.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.magenta.value)
  
  def test_irt_requirement_0090_006(self):
    '''Testing BlackBackground with AlphaRed'''
    with open("src/test/resources/stl/irt/requirement-0090-006.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.red.value)
  
  def test_irt_requirement_0090_007(self):
    '''Testing BlackBackground with AlphaWhite'''
    with open("src/test/resources/stl/irt/requirement-0090-007.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.white.value)
  
  def test_irt_requirement_0090_008(self):
    '''Testing BlackBackground with AlphaYellow'''
    with open("src/test/resources/stl/irt/requirement-0090-008.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.yellow.value)
  
  def test_irt_requirement_0090_009(self):
    '''Testing unchanging BlackBackground'''
    with open("src/test/resources/stl/irt/requirement-0090-009.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.white.value)

  def test_irt_requirement_0091_001(self):
    '''Testing NewBackground with AlphaBlack'''
    with open("src/test/resources/stl/irt/requirement-0091-001.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)
  
  def test_irt_requirement_0091_002(self):
    '''Testing NewBackground with AlphaBlue'''
    with open("src/test/resources/stl/irt/requirement-0091-002.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_first_span = spans[0].get_style(styles.StyleProperties.BackgroundColor)
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_first_span, styles.NamedColors.black.value)
      self.assertEqual(background_color_second_span, styles.NamedColors.blue.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)
  
  def test_irt_requirement_0091_003(self):
    '''Testing NewBackground with AlphaCyan'''
    with open("src/test/resources/stl/irt/requirement-0091-003.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.cyan.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)
  
  def test_irt_requirement_0091_004(self):
    '''Testing NewBackground with AlphaGreen'''
    with open("src/test/resources/stl/irt/requirement-0091-004.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.lime.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)
  
  def test_irt_requirement_0091_005(self):
    '''Testing NewBackground with AlphaMagenta'''
    with open("src/test/resources/stl/irt/requirement-0091-005.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.magenta.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)

  def test_irt_requirement_0091_006(self):
    '''Testing NewBackground with AlphaRed'''
    with open("src/test/resources/stl/irt/requirement-0091-006.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.red.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)   
  
  def test_irt_requirement_0091_007(self):
    '''Testing NewBackground with AlphaWhite'''
    with open("src/test/resources/stl/irt/requirement-0091-007.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.white.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)
  
  def test_irt_requirement_0091_008(self):
    '''Testing NewBackground with AlphaYellow'''
    with open("src/test/resources/stl/irt/requirement-0091-008.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.yellow.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)
  
  def test_irt_requirement_0091_009(self):
    '''Testing unchanging NewBackground'''
    with open("src/test/resources/stl/irt/requirement-0091-009.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_second_span = spans[1].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = spans[1].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.lime.value)
      self.assertEqual(color_second_span, styles.NamedColors.black.value)

  def test_setting_background_before_startbox(self):
    '''Testing NewBackground Control Code before StartBox'''    
    with open("src/test/resources/stl/sandflow/setting_background_before_startbox.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      spans = list(doc.get_body().first_child().first_child())
      background_color_first_span = spans[0].get_style(styles.StyleProperties.BackgroundColor)
      self.assertEqual(background_color_first_span, styles.NamedColors.yellow.value)

  def test_multi_tti_subtitle(self):
    '''Testing subtitle that spans over multiple TTI blocks with different EBN values'''
    with open("src/test/resources/stl/sandflow/multi_tti_subtitle.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      text = doc.get_body()\
                .first_child()\
                .first_child()\
                .first_child()\
                .first_child()\
                .get_text()
      self.assertEqual(text, "Foo Bar Baz")

  def test_default_font_stack(self):
    '''Testing default font stack'''  
    with open("src/test/resources/stl/sandflow/setting_background_before_startbox.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      self.assertEqual(doc.get_body().get_style(
        styles.StyleProperties.FontFamily),
        ("Verdana", "Arial", "Tiresias", styles.GenericFontFamilyType.sansSerif)
      )

  def test_cumulative_set(self):
    '''Testing cumulative subtitles'''
    with open("src/test/resources/stl/sandflow/cumulative_set.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p_list = list(doc.get_body().first_child())
      p1_span = p_list[0].first_child()
      self.assertEqual(p1_span.first_child().get_text(),
                       "Not part of cumulative set.")
      
      p2_children = list(p_list[1])
      self.assertEqual(len(p2_children), 4)

      span1_text = p2_children[0].first_child().first_child().get_text()
      span2_text = p2_children[1].first_child().first_child().get_text()
      span3_text = p2_children[2].first_child().first_child().get_text()
      span4_text = p2_children[3].first_child().first_child().get_text()

      self.assertEqual(span1_text, "1")
      self.assertEqual(span2_text, "2")
      self.assertEqual(span3_text, "3")
      self.assertEqual(span4_text, "4")
  
  def test_vp18_3_lines(self):
    '''Testing vertical positioning of multi-row subtitle'''
    with open("src/test/resources/stl/sandflow/vp18_3_lines.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      region = p.get_region()
      region_extent = region.get_style(styles.StyleProperties.Extent).height.value
      self.assertEqual(region_extent, 80)

  def test_vp20_2_newlines(self):
    '''Testing vertical positioning with two newlines between double height rows'''
    with open("src/test/resources/stl/sandflow/vp20_2_newlines.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p = doc.get_body().first_child().first_child()
      region = p.get_region()
      region_extent = region.get_style(styles.StyleProperties.Extent).height.value
      self.assertEqual(region_extent, 80)

  def test_br_style_reset(self):
    '''Testing style reset after newline'''
    with open("src/test/resources/stl/sandflow/br_style_reset.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p_children = list(doc.get_body().first_child().first_child())
      background_color_second_span = p_children[2].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = p_children[2].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.black.value)
      self.assertEqual(color_second_span, styles.NamedColors.white.value)

  def test_br_new_colors(self):
    '''Testing new color settings after newline'''
    with open("src/test/resources/stl/sandflow/br_new_colors.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p_children = list(doc.get_body().first_child().first_child())
      background_color_second_span = p_children[2].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = p_children[2].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.blue.value)
      self.assertEqual(color_second_span, styles.NamedColors.yellow.value)

  def test_br_same_colors(self):
    '''Testing color settings after newline, colors do not change'''
    with open("src/test/resources/stl/sandflow/br_same_colors.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      p_children = list(doc.get_body().first_child().first_child())
      background_color_second_span = p_children[2].get_style(styles.StyleProperties.BackgroundColor)
      color_second_span = p_children[2].get_style(styles.StyleProperties.Color)
      self.assertEqual(background_color_second_span, styles.NamedColors.magenta.value)
      self.assertEqual(color_second_span, styles.NamedColors.yellow.value)

  def test_overlapping_tti(self):
    '''TTI blocks with overlapping active periods: second TCI is adjusted to equal the first TCO'''
    with open("src/test/resources/stl/sandflow/overlapping_tti.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      div = doc.get_body().first_child()
      p1 = div.first_child()
      p2 = p1.next_sibling()
      self.assertEqual(p1.get_begin(), SmpteTimeCode.parse("00:00:01:00", FPS_25).to_temporal_offset())
      self.assertEqual(p1.get_end(), SmpteTimeCode.parse("00:00:05:00", FPS_25).to_temporal_offset())
      self.assertEqual(p2.get_begin(), SmpteTimeCode.parse("00:00:05:00", FPS_25).to_temporal_offset())
      self.assertEqual(p2.get_end(), SmpteTimeCode.parse("00:00:07:00", FPS_25).to_temporal_offset())
      self.assertEqual(p1.first_child().first_child().get_text(), "Subtitle One")
      self.assertEqual(p2.first_child().first_child().get_text(), "Subtitle Two")

  def test_contained_tti(self):
    '''Second TTI block active period contained within the first: second subtitle is dropped'''
    with open("src/test/resources/stl/sandflow/contained_tti.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      div = doc.get_body().first_child()
      ps = list(div)
      self.assertEqual(len(ps), 1)
      self.assertEqual(ps[0].get_begin(), SmpteTimeCode.parse("00:00:01:00", FPS_25).to_temporal_offset())
      self.assertEqual(ps[0].get_end(), SmpteTimeCode.parse("00:00:07:00", FPS_25).to_temporal_offset())
      self.assertEqual(ps[0].first_child().first_child().get_text(), "Subtitle One")

  def test_two_contained_tti(self):
    '''Two TTI blocks with active periods contained within a third: both inner subtitles are dropped'''
    with open("src/test/resources/stl/sandflow/two_contained_tti.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      div = doc.get_body().first_child()
      ps = list(div)
      self.assertEqual(len(ps), 1)
      self.assertEqual(ps[0].get_begin(), SmpteTimeCode.parse("00:00:01:00", FPS_25).to_temporal_offset())
      self.assertEqual(ps[0].get_end(), SmpteTimeCode.parse("00:00:09:00", FPS_25).to_temporal_offset())
      self.assertEqual(ps[0].first_child().first_child().get_text(), "Subtitle One")

  def test_active_area_presence(self):
    with open("src/test/resources/stl/irt/requirement-0056-001_modified.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      
      self.assertIsNotNone(doc.get_active_area())

  def test_cell_resolution_presence(self):
    with open("src/test/resources/stl/irt/requirement-0056-001_modified.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      
      self.assertIsNotNone(doc.get_cell_resolution())

  def test_content_profiles_presence(self):
    with open("src/test/resources/stl/irt/requirement-0056-001_modified.stl", "rb") as f:
      doc = ttconv.stl.reader.to_model(f)
      self.assertSetEqual({IMSC_11_TEXT_PROFILE_DESIGNATOR}, doc.get_content_profiles())

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""TT configuration"""

from __future__ import annotations

import dataclasses
from dataclasses import dataclass
from typing import Optional, Dict, List, Any


class ModuleConfiguration:
  """Base class for module configurations"""

  @classmethod
  def get_fields(cls) -> List[dataclasses.Field]:
    """Returns data class fields"""
    return list(dataclasses.fields(cls))

  @classmethod
  def validate(cls, config_dict: Dict):
    """Validates configuration dictionary"""
    for field in cls.get_fields():
      optional_field = "Optional" in field.type or cls.get_field_default(field) is not None

      config_value = config_dict.get(field.name)
      if not optional_field and config_value is None:
        raise ValueError("Compulsory configuration field missing:", field.name)

  @classmethod
  def parse(cls, config_dict: Dict) -> ModuleConfiguration:
    """Parses configuration dictionary"""
    cls.validate(config_dict)

    kwargs = {}
    for field in cls.get_fields():

      field_value = config_dict.get(field.name, cls.get_field_default(field))

      decoder = field.metadata.get("decoder")
      if decoder is not None:
        field_value = decoder.__call__(field_value)

      kwargs[field.name] = field_value

    instance = cls(**kwargs)

    return instance

  @staticmethod
  def get_field_default(field: dataclasses.Field) -> Optional[Any]:
    """Returns the default field value if any, None otherwise"""
    if isinstance(field.default, dataclasses._MISSING_TYPE):
      return None
    return field.default

  @classmethod
  def name(cls):
    """Returns the configuration name"""
    raise NotImplementedError


@dataclass
class GeneralConfiguration(ModuleConfiguration):
  """TT general configuration"""
  log_level: Optional[str] = "INFO"
  progress_bar: Optional[bool] = True
  document_lang: Optional[str] = None

  @classmethod
  def name(cls):
    return "general"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SCC Codes"""

from enum import Enum
from typing import Tuple, Optional
from ttconv.style_properties import NamedColors

SCC_COLOR_MAPPING = {
  0x00: NamedColors.white.value,
  0x01: NamedColors.white.value,
  0x02: NamedColors.green.value,
  0x03: NamedColors.green.value,
  0x04: NamedColors.blue.value,
  0x05: NamedColors.blue.value,
  0x06: NamedColors.cyan.value,
  0x07: NamedColors.cyan.value,
  0x08: NamedColors.red.value,
  0x09: NamedColors.red.value,
  0x0A: NamedColors.yellow.value,
  0x0B: NamedColors.yellow.value,
  0x0C: NamedColors.magenta.value,
  0x0D: NamedColors.magenta.value
}

class SccChannel(Enum):
  """SCC Caption Channel"""
  CHANNEL_1 = 1
  CHANNEL_2 = 2

  def __str__(self):
    return "CC" + str(self.value)


class SccCode(Enum):
  """SCC codes base definition class"""

  def __init__(self, channel_1: int, channel_2: int):
    if channel_1 > 0xFFFF or channel_2 > 0xFFFF:
      raise ValueError("Expected 2-bytes values")

    self._channel_1 = channel_1
    self._channel_2 = channel_2

  def get_values(self) -> Tuple[int, int]:
    """Returns SCC Code values"""
    return self._channel_1, self._channel_2

  def get_ch1_value(self) -> int:
    """Returns Channel 1 Code Value"""
    return self._channel_1

  def get_channel(self, value: int) -> Optional[SccChannel]:
    """Returns caption channel corresponding to the specified code value"""
    if value == self._channel_1:
      return SccChannel.CHANNEL_1

    if value == self._channel_2:
      return SccChannel.CHANNEL_2

    return None

  def contains_value(self, value: int) -> bool:
    """Returns whether the specified value is contained into the SCC code channels values"""
    return value in self.get_values()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2021, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""WebVTT paragraph"""

from enum import Enum
import re
from fractions import Fraction
from typing import Optional, Union

from ttconv.time_code import ClockTime


class VttCue:
  """VTT cue class"""

  class LineAlignment(Enum):
    """WebVTT line alignment cue setting"""
    start = "start"
    center = "center"
    end = "end"

  class TextAlignment(Enum):
    """WebVTT text alignment cue setting"""
    left = "left"
    center = "center"
    right = "right"

  _EOL_SEQ_RE = re.compile(r"\n{2,}")

  def __init__(self, identifier: Optional[int] = None):
    self._id: int = identifier
    self._begin: Optional[ClockTime] = None
    self._end: Optional[ClockTime] = None
    self._text: str = ""
    self._line: int = None
    self._linealign: VttCue.LineAlignment = None
    self._textalign: VttCue.TextAlignment = None

  def set_begin(self, offset: Fraction):
    """Sets the paragraph begin time code"""
    self._begin = ClockTime.from_seconds(offset)
    self._begin.set_separator(".")

  def get_begin(self) -> Optional[ClockTime]:
    """Returns the paragraph begin time code"""
    return self._begin

  def set_end(self, offset: Optional[Union[Fraction, float]]):
    """Sets the paragraph end time code"""
    if offset is not None:
      self._end = ClockTime.from_seconds(offset)
      self._end.set_separator(".")
    else:
      self._end = None

  def get_end(self) -> Optional[ClockTime]:
    """Returns the paragraph end time code"""
    return self._end

  def set_line(self, line: int):
    """Sets the WebVTT line cue setting (in whole percent)"""
    self._line = line

  def get_line(self) -> Optional[int]:
    """Return the WebVTT line cue setting (in whole percent)"""
    return self._line

  def set_align(self, align: LineAlignment):
    """Sets the WebVTT line alignment cue setting"""
    self._linealign = align

  def get_align(self) -> Optional[LineAlignment]:
    """Return the WebVTT line alignment cue setting"""
    return self._linealign

  def set_textalign(self, textalign: TextAlignment):
    """Sets the WebVTT text alignment cue setting"""
    self._textalign = textalign

  def is_only_whitespace_or_empty(self):
    """Returns whether the paragraph text contains only whitespace or is empty"""
    return len(self._text) == 0 or self._text.isspace()

  def normalize_eol(self):
    """Remove line breaks at the beginning and end of the paragraph, and replace
    line break sequences with a single line break"""
    self._text = VttCue._EOL_SEQ_RE.sub("\n", self._text).strip("\n\r")

  def append_text(self, text: str):
    """Appends text to the paragraph"""
    self._text += text

  def to_string(self) -> str:
    """Returns the VTT paragraph as a formatted string"""
    if self._begin is None:
      raise ValueError("VTT paragraph begin time code must be set.")

    if self._end is None:
      raise ValueError("VTT paragraph end time code must be set.")

    if self._end.to_seconds() <= self._begin.to_seconds():
      raise ValueError("VTT paragraph end time code must be greater than the begin time code.")

    return str(self)

  def __str__(self) -> str:

    # cue identifier
    t = f"{self._id}\n" if self._id is not None else ""
    
    # cue timing
    t += f"{self._begin} --> {self._end}"

    # cue text position
    if self._textalign is not None:
      t += f" align:{self._textalign.value}"

    # cue line position
    if self._line is not None:
      t += f" line:{self._line}%"

      if self._linealign is not None:
        t += f",{self._linealign.value}"

    t += "\n"

    # cue body
    if self._text:
      t += self._text + "\n"

    return t



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SRT writer"""

import logging
from fractions import Fraction
from typing import List, Optional

import ttconv.model as model
import ttconv.srt.style as style
from ttconv.filters.isd_filter import ISDFilter
from ttconv.filters.isd.default_style_properties import DefaultStylePropertyValuesISDFilter
from ttconv.filters.isd.merge_paragraphs import ParagraphsMergingISDFilter
from ttconv.filters.isd.merge_regions import RegionsMergingISDFilter
from ttconv.filters.isd.supported_style_properties import SupportedStylePropertiesISDFilter
from ttconv.isd import ISD
from ttconv.srt.paragraph import SrtParagraph
from ttconv.srt.config import SRTWriterConfiguration
from ttconv.style_properties import StyleProperties, FontStyleType, NamedColors, FontWeightType, TextDecorationType

LOGGER = logging.getLogger(__name__)


class SrtContext:
  """SRT writer context"""

  filters: List[ISDFilter] = (
    RegionsMergingISDFilter(),
    ParagraphsMergingISDFilter(),
    SupportedStylePropertiesISDFilter({
      StyleProperties.FontWeight: [
        # Every values
      ],
      StyleProperties.FontStyle: [
        FontStyleType.normal,
        FontStyleType.italic
      ],
      StyleProperties.TextDecoration: [
        TextDecorationType.underline
      ],
      StyleProperties.Color: [
        # Every values
      ],
    }),
    DefaultStylePropertyValuesISDFilter({
      StyleProperties.Color: NamedColors.white.value,
      StyleProperties.FontWeight: FontWeightType.normal,
      StyleProperties.FontStyle: FontStyleType.normal,
    })
  )

  def __init__(self, config: SRTWriterConfiguration):
    self._captions_counter: int = 0
    self._begin: Fraction = Fraction(0)
    self._end: Fraction = Fraction(0)
    self._paragraphs: List[SrtParagraph] = []
    self._text_formatting = config.text_formatting

  def append_element(self, element: model.ContentElement, begin: Fraction, end: Optional[Fraction]):
    """Converts model element to SRT content"""

    if isinstance(element, model.Div):
      for elem in list(element):
        self.append_element(elem, begin, end)

    if isinstance(element, model.P):

      self._captions_counter += 1

      self._paragraphs.append(SrtParagraph(self._captions_counter))
      self._paragraphs[-1].set_begin(begin)
      self._paragraphs[-1].set_end(end)

      for elem in list(element):
        self.append_element(elem, begin, end)

      self._paragraphs[-1].normalize_eol()

      if self._paragraphs[-1].is_only_whitespace():
        LOGGER.debug("Removing empty paragraph.")
        self._paragraphs.pop()

    if isinstance(element, model.Span):
      is_bold = style.is_element_bold(element)
      is_italic = style.is_element_italic(element)
      is_underlined = style.is_element_underlined(element)
      font_color = style.get_font_color(element)

      if self._text_formatting:
        if font_color is not None:
          self._paragraphs[-1].append_text(style.FONT_COLOR_TAG_IN.format(font_color))
        if is_bold:
          self._paragraphs[-1].append_text(style.BOLD_TAG_IN)
        if is_italic:
          self._paragraphs[-1].append_text(style.ITALIC_TAG_IN)
        if is_underlined:
          self._paragraphs[-1].append_text(style.UNDERLINE_TAG_IN)

      for elem in list(element):
        self.append_element(elem, begin, end)

      if self._text_formatting:
        if is_underlined:
          self._paragraphs[-1].append_text(style.UNDERLINE_TAG_OUT)
        if is_italic:
          self._paragraphs[-1].append_text(style.ITALIC_TAG_OUT)
        if is_bold:
          self._paragraphs[-1].append_text(style.BOLD_TAG_OUT)
        if font_color is not None:
          self._paragraphs[-1].append_text(style.FONT_COLOR_TAG_OUT)

    if isinstance(element, model.Br):
      self._paragraphs[-1].append_text("\n")

    if isinstance(element, model.Text):
      self._paragraphs[-1].append_text(element.get_text())

  def add_isd(self, isd, begin: Fraction, end: Optional[Fraction]):
    """Converts and appends ISD content to SRT content"""

    LOGGER.debug(
      "Append ISD from %ss to %ss to SRT content.",
      float(begin),
      float(end) if end is not None else "unbounded"
    )

    is_isd_empty = True

    for region in isd.iter_regions():

      if len(region) > 0:
        is_isd_empty = False

      for body in region:
        for div in list(body):
          self.append_element(div, begin, end)

    if is_isd_empty:
      LOGGER.debug("Skipping empty paragraph.")

  def finish(self):
    """Checks and processes the last paragraph"""

    LOGGER.debug("Check and process the last SRT paragraph.")

    if self._paragraphs and self._paragraphs[-1].get_end() is None:
      if self._paragraphs[-1].is_only_whitespace():
        # if the last paragraph contains only whitespace, remove it
        LOGGER.debug("Removing empty unbounded last paragraph.")
        self._paragraphs.pop()

      else:
        # set default end time code
        LOGGER.warning("Set a default end value to paragraph (begin + 10s).")
        self._paragraphs[-1].set_end(self._paragraphs[-1].get_begin().to_seconds() + 10.0)

  def __str__(self) -> str:
    return "\n".join(p.to_string(id + 1) for id, p in enumerate(self._paragraphs))


#
# srt writer
#


def from_model(doc: model.ContentDocument, config: Optional[SRTWriterConfiguration] = None, progress_callback=lambda _: None) -> str:
  """Converts the data model to a SRT document"""

  srt = SrtContext(config if config is not None else SRTWriterConfiguration())

  # split progress between ISD construction and SRT writing

  def _isd_progress(progress: float):
    progress_callback(progress / 2)

  # Compute ISDs

  isds = list(
    ISD.generate_isd_sequence(doc, _isd_progress)
  )

  # process ISDs

  for i, (begin, isd) in enumerate(isds):

    end = isds[i + 1][0] if i + 1 < len(isds) else None

    for srt_filter in srt.filters:
      srt_filter.process(isd)

    srt.add_isd(isd, begin, end)

    progress_callback(0.5 + (i + 1) / len(isds) / 2)

  srt.finish()

  return str(srt)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC Standard characters mapping"""

# pylint: disable=R0201,C0115,C0116,W0212

import unittest

from ttconv.scc.codes.characters import unicode_to_scc
from ttconv.scc.codes.standard_characters import SCC_STANDARD_CHARACTERS_MAPPING


class SccStandardCharactersTest(unittest.TestCase):

  def test_unicode_to_scc_standard_character(self):
    for scc_char, unicode_char in SCC_STANDARD_CHARACTERS_MAPPING.items():
      with self.subTest(unicode_char=unicode_char):
        self.assertEqual(unicode_to_scc(unicode_char)[0], scc_char)

  def test_scc_standard_character_values(self):
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x20], " ")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x21], "!")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x22], '"')
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x23], "#")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x24], "$")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x25], "%")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x26], "&")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x27], "'")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x28], "(")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x29], ")")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x2A], "á")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x2B], "+")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x2C], ",")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x2D], "-")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x2E], ".")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x2F], "/")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x30], "0")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x31], "1")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x32], "2")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x33], "3")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x34], "4")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x35], "5")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x36], "6")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x37], "7")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x38], "8")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x39], "9")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x3A], ":")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x3B], ";")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x3C], "<")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x3D], "=")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x3E], ">")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x3F], "?")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x40], "@")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x41], "A")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x42], "B")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x43], "C")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x44], "D")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x45], "E")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x46], "F")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x47], "G")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x48], "H")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x49], "I")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x4A], "J")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x4B], "K")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x4C], "L")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x4D], "M")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x4E], "N")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x4F], "O")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x50], "P")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x51], "Q")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x52], "R")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x53], "S")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x54], "T")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x55], "U")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x56], "V")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x57], "W")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x58], "X")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x59], "Y")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x5A], "Z")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x5B], "[")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x5C], "é")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x5D], "]")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x5E], "í")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x5F], "ó")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x60], "ú")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x61], "a")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x62], "b")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x63], "c")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x64], "d")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x65], "e")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x66], "f")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x67], "g")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x68], "h")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x69], "i")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x6A], "j")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x6B], "k")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x6C], "l")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x6D], "m")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x6E], "n")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x6F], "o")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x70], "p")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x71], "q")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x72], "r")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x73], "s")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x74], "t")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x75], "u")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x76], "v")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x77], "w")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x78], "x")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x79], "y")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x7A], "z")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x7B], "ç")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x7C], "÷")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x7D], "Ñ")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x7E], "ñ")
    self.assertEqual(SCC_STANDARD_CHARACTERS_MAPPING[0x7F], "█")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''Unit tests for the IMSC \\<position\\> parser'''

# pylint: disable=R0201,C0115,C0116

import unittest
from ttconv.imsc.utils import parse_position
import ttconv.style_properties as styles

def position(h_edge, h_offval, h_offunit, v_edge, v_offval, v_offunit):
  return (
    h_edge,
    styles.LengthType(value=h_offval, units=styles.LengthType.Units(h_offunit)),
    v_edge,
    styles.LengthType(value=v_offval, units=styles.LengthType.Units(v_offunit))
  )

class IMSCPositionTest(unittest.TestCase):

  tests = [
    # one component
    ["center", position("left", 50, "%", "top", 50, "%")],
    ["left", position("left", 0, "%", "top", 50, "%")],
    ["right", position("right", 0, "%", "top", 50, "%")],
    ["top", position("left", 50, "%", "top", 0, "%")],
    ["bottom", position("left", 50, "%", "bottom", 0, "%")],
    ["25%", position("left", 25, "%", "top", 50, "%")],
    # two components
    ["bottom center", position("left", 50, "%", "bottom", 0, "%")],
    ["bottom left", position("left", 0, "%", "bottom", 0, "%")],
    ["bottom right", position("right", 0, "%", "bottom", 0, "%")],
    ["center center", position("left", 50, "%", "top", 50, "%")],
    ["center top", position("left", 50, "%", "top", 0, "%")],
    ["center bottom", position("left", 50, "%", "bottom", 0, "%")],
    ["center left", position("left", 0, "%", "top", 50, "%")],
    ["center right", position("right", 0, "%", "top", 50, "%")],
    ["center 33%", position("left", 50, "%", "top", 33, "%")],
    ["left center", position("left", 0, "%", "top", 50, "%")],
    ["left top", position("left", 0, "%", "top", 0, "%")],
    ["left bottom", position("left", 0, "%", "bottom", 0, "%")],
    ["left 45%", position("left", 0, "%", "top", 45, "%")],
    ["right center", position("right", 0, "%", "top", 50, "%")],
    ["right top", position("right", 0, "%", "top", 0, "%")],
    ["right bottom", position("right", 0, "%", "bottom", 0, "%")],
    ["right 20%", position("right", 0, "%", "top", 20, "%")],
    ["top center", position("left", 50, "%", "top", 0, "%")],
    ["top left", position("left", 0, "%", "top", 0, "%")],
    ["top right", position("right", 0, "%", "top", 0, "%")],
    ["75% center", position("left", 75, "%", "top", 50, "%")],
    ["75% top", position("left", 75, "%", "top", 0, "%")],
    ["75% bottom", position("left", 75, "%", "bottom", 0, "%")],
    ["75% 75%", position("left", 75, "%", "top", 75, "%")],
    # three components
    ["bottom left 1%", position("left", 1, "%", "bottom", 0, "%")],
    ["bottom right 2%", position("right", 2, "%", "bottom", 0, "%")],
    ["bottom 3% center", position("left", 50, "%", "bottom", 3, "%")],
    ["bottom 4% left", position("left", 0, "%", "bottom", 4, "%")],
    ["bottom 5% right", position("right", 0, "%", "bottom", 5, "%")],
    ["center bottom 6%", position("left", 50, "%", "bottom", 6, "%")],
    ["center left 7%", position("left", 7, "%", "top", 50, "%")],
    ["center right 8%", position("right", 8, "%", "top", 50, "%")],
    ["center top 9%", position("left", 50, "%", "top", 9, "%")],
    ["left bottom 10%", position("left", 0, "%", "bottom", 10, "%")],
    ["left top 11%", position("left", 0, "%", "top", 11, "%")],
    ["left 12% bottom", position("left", 12, "%", "bottom", 0, "%")],
    ["left 13% center", position("left", 13, "%", "top", 50, "%")],
    ["left 14% top", position("left", 14, "%", "top", 0, "%")],
    ["right bottom 15%", position("right", 0, "%", "bottom", 15, "%")],
    ["right top 16%", position("right", 0, "%", "top", 16, "%")],
    ["right 17% bottom", position("right", 17, "%", "bottom", 0, "%")],
    ["right 18% center", position("right", 18, "%", "top", 50, "%")],
    ["right 19% top", position("right", 19, "%", "top", 0, "%")],
    ["top left 20%", position("left", 20, "%", "top", 0, "%")],
    ["top right 21%", position("right", 21, "%", "top", 0, "%")],
    ["top 22% center", position("left", 50, "%", "top", 22, "%")],
    ["top 23% left", position("left", 0, "%", "top", 23, "%")],
    ["top 24% right", position("right", 0, "%", "top", 24, "%")],
    # four components
    ["bottom 25% left 75%", position("left", 75, "%", "bottom", 25, "%")],
    ["bottom 25% right 75%", position("right", 75, "%", "bottom", 25, "%")],
    ["left 25% bottom 75%", position("left", 25, "%", "bottom", 75, "%")],
    ["right 25% bottom 75%", position("right", 25, "%", "bottom", 75, "%")],
    ["top 25% left 75%", position("left", 75, "%", "top", 25, "%")],
    ["top 25% right 75%", position("right", 75, "%", "top", 25, "%")],
    ["left 25% top 75%", position("left", 25, "%", "top", 75, "%")],
    ["right 25% top 75%", position("right", 25, "%", "top", 75, "%")]
  ]

  def test_positions(self):
    for test in self.tests:
      with self.subTest(test[0]):
        c = parse_position(test[0])
        self.assertEqual(c, test[1])

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC Control codes"""

# pylint: disable=R0201,C0115,C0116

import unittest

from ttconv.scc.codes.control_codes import SccControlCode

FIELD_1_CONTROL_CODE_VALUES = [0X1422, 0x1C22, 0X1423, 0x1C23, 0X1421, 0x1C21, 0X142D, 0x1C2D, 0X1424,
                               0x1C24, 0X142C, 0x1C2C, 0X142E, 0x1C2E, 0X142F, 0x1C2F, 0X1428, 0x1C28,
                               0X1429, 0x1C29, 0X142B, 0x1C2B, 0X1721, 0x1F21, 0X1722, 0x1F22, 0X1723,
                               0x1F23, 0X142A, 0x1C2A, 0X1420, 0x1C20, 0X1425, 0x1C25, 0X1426, 0x1C26,
                               0X1427, 0x1C27]

FIELD_2_CONTROL_CODE_VALUES = [0x1522, 0x1D22, 0x1523, 0x1D23, 0x1521, 0x1D21, 0x152D, 0x1D2D, 0x1524, 0x1D24,
                               0x152C, 0x1D2C, 0x152E, 0x1D2E, 0x152F, 0x1D2F, 0x1528, 0x1D28, 0x1529, 0x1D29,
                               0x152B, 0x1D2B, 0x1721, 0x1F21, 0x1722, 0x1F22, 0x1723, 0x1F23, 0x152A, 0x1D2A,
                               0x1520, 0x1D20, 0x1525, 0x1D25, 0x1526, 0x1D26, 0x1527, 0x1D27]


class SCCControlCodesTest(unittest.TestCase):

  def test_scc_control_codes(self):
    self.assertEqual(SccControlCode.AOF,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[0]))
    self.assertEqual(SccControlCode.AOF,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[0]))
    self.assertEqual(SccControlCode.AOF,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[1]))
    self.assertEqual(SccControlCode.AOF,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[1]))
    self.assertEqual(SccControlCode.AON,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[2]))
    self.assertEqual(SccControlCode.AON,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[2]))
    self.assertEqual(SccControlCode.AON,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[3]))
    self.assertEqual(SccControlCode.AON,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[3]))
    self.assertEqual(SccControlCode.BS,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[4]))
    self.assertEqual(SccControlCode.BS,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[4]))
    self.assertEqual(SccControlCode.BS,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[5]))
    self.assertEqual(SccControlCode.BS,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[5]))
    self.assertEqual(SccControlCode.CR,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[6]))
    self.assertEqual(SccControlCode.CR,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[6]))
    self.assertEqual(SccControlCode.CR,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[7]))
    self.assertEqual(SccControlCode.CR,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[7]))
    self.assertEqual(SccControlCode.DER,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[8]))
    self.assertEqual(SccControlCode.DER,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[8]))
    self.assertEqual(SccControlCode.DER,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[9]))
    self.assertEqual(SccControlCode.DER,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[9]))
    self.assertEqual(SccControlCode.EDM,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[10]))
    self.assertEqual(SccControlCode.EDM,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[10]))
    self.assertEqual(SccControlCode.EDM,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[11]))
    self.assertEqual(SccControlCode.EDM,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[11]))
    self.assertEqual(SccControlCode.ENM,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[12]))
    self.assertEqual(SccControlCode.ENM,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[12]))
    self.assertEqual(SccControlCode.ENM,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[13]))
    self.assertEqual(SccControlCode.ENM,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[13]))
    self.assertEqual(SccControlCode.EOC,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[14]))
    self.assertEqual(SccControlCode.EOC,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[14]))
    self.assertEqual(SccControlCode.EOC,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[15]))
    self.assertEqual(SccControlCode.EOC,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[15]))
    self.assertEqual(SccControlCode.FON,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[16]))
    self.assertEqual(SccControlCode.FON,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[16]))
    self.assertEqual(SccControlCode.FON,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[17]))
    self.assertEqual(SccControlCode.FON,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[17]))
    self.assertEqual(SccControlCode.RDC,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[18]))
    self.assertEqual(SccControlCode.RDC,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[18]))
    self.assertEqual(SccControlCode.RDC,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[19]))
    self.assertEqual(SccControlCode.RDC,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[19]))
    self.assertEqual(SccControlCode.RTD,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[20]))
    self.assertEqual(SccControlCode.RTD,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[20]))
    self.assertEqual(SccControlCode.RTD,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[21]))
    self.assertEqual(SccControlCode.RTD,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[21]))
    self.assertEqual(SccControlCode.TO1,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[22]))
    self.assertEqual(SccControlCode.TO1,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[22]))
    self.assertEqual(SccControlCode.TO1,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[23]))
    self.assertEqual(SccControlCode.TO1,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[23]))
    self.assertEqual(SccControlCode.TO2,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[24]))
    self.assertEqual(SccControlCode.TO2,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[24]))
    self.assertEqual(SccControlCode.TO2,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[25]))
    self.assertEqual(SccControlCode.TO2,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[25]))
    self.assertEqual(SccControlCode.TO3,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[26]))
    self.assertEqual(SccControlCode.TO3,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[26]))
    self.assertEqual(SccControlCode.TO3,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[27]))
    self.assertEqual(SccControlCode.TO3,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[27]))
    self.assertEqual(SccControlCode.TR,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[28]))
    self.assertEqual(SccControlCode.TR,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[28]))
    self.assertEqual(SccControlCode.TR,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[29]))
    self.assertEqual(SccControlCode.TR,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[29]))
    self.assertEqual(SccControlCode.RCL,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[30]))
    self.assertEqual(SccControlCode.RCL,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[30]))
    self.assertEqual(SccControlCode.RCL,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[31]))
    self.assertEqual(SccControlCode.RCL,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[31]))
    self.assertEqual(SccControlCode.RU2,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[32]))
    self.assertEqual(SccControlCode.RU2,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[32]))
    self.assertEqual(SccControlCode.RU2,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[33]))
    self.assertEqual(SccControlCode.RU2,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[33]))
    self.assertEqual(SccControlCode.RU3,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[34]))
    self.assertEqual(SccControlCode.RU3,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[34]))
    self.assertEqual(SccControlCode.RU3,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[35]))
    self.assertEqual(SccControlCode.RU3,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[35]))
    self.assertEqual(SccControlCode.RU4,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[36]))
    self.assertEqual(SccControlCode.RU4,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[36]))
    self.assertEqual(SccControlCode.RU4,
                     SccControlCode.find(FIELD_1_CONTROL_CODE_VALUES[37]))
    self.assertEqual(SccControlCode.RU4,
                     SccControlCode.find(FIELD_2_CONTROL_CODE_VALUES[37]))

  def test_scc_control_codes_invalid(self):
    other_code_values = [code for code in range(0x0000, 0xFFFF) if
                         (code not in FIELD_1_CONTROL_CODE_VALUES) and code not in FIELD_2_CONTROL_CODE_VALUES]

    for cc in other_code_values:
      self.assertIsNone(SccControlCode.find(cc))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SCC context"""

from __future__ import annotations

import copy
import logging
from typing import Optional, Type, Tuple

from ttconv.model import Div
from ttconv.scc.caption_paragraph import SccCaptionParagraph
from ttconv.scc.caption_style import SccCaptionStyle
from ttconv.scc.codes import SccChannel
from ttconv.scc.codes.attribute_codes import SccAttributeCode
from ttconv.scc.codes.control_codes import SccControlCode
from ttconv.scc.codes.mid_row_codes import SccMidRowCode
from ttconv.scc.codes.preambles_address_codes import SccPreambleAddressCode
from ttconv.scc.config import SccReaderConfiguration, TextAlignment
from ttconv.scc.word import SccWord
from ttconv.style_properties import StyleProperties
from ttconv.time_code import SmpteTimeCode

ROLL_UP_BASE_ROW = 15

LOGGER = logging.getLogger(__name__)


class SccContext:
  """SCC context for reader"""

  def __init__(self, safe_area_x_offset: int, safe_area_y_offset: int, config: Optional[SccReaderConfiguration] = None):
    # Caption paragraphs container
    self.div: Optional[Div] = None

    # Caption paragraphs counter
    self.count: int = 0

    # Screen safe area offsets
    self.safe_area_x_offset = safe_area_x_offset
    self.safe_area_y_offset = safe_area_y_offset

    # Previously read SCC word value
    self.previous_word: Optional[SccWord] = None
    self.previous_word_type: Optional[Type] = None

    # Caption style (Pop-on, Roll-up, Paint-on) currently processed
    self.current_style = SccCaptionStyle.default()
    # Buffered caption being built
    self.buffered_caption = None
    # Captions being displayed
    self.active_caption: Optional[SccCaptionParagraph] = None

    # Current caption channel (default is CC1)
    self.current_channel = SccChannel.CHANNEL_1

    # Roll-up caption number of lines
    self.roll_up_depth: int = 0

    # Cursor position in the active area
    self.active_cursor: Tuple[int, int] = (0, 0)

    self.current_text_decoration = None
    self.current_color = None
    self.current_font_style = None

    # Text alignment
    self.text_alignment = TextAlignment.AUTO if config is None else config.text_align

    self.new_buffered_caption()

  def new_active_caption(self, begin_time_code: SmpteTimeCode, caption_style: SccCaptionStyle = SccCaptionStyle.Unknown):
    """Initializes a new caption being displayed"""
    self.active_caption = SccCaptionParagraph(self.safe_area_x_offset, self.safe_area_y_offset, caption_style)
    self.count += 1
    self.active_caption.set_id("caption" + str(self.count))
    self.active_caption.set_begin(begin_time_code)

  def new_buffered_caption(self):
    """Resets buffered caption"""
    self.buffered_caption = SccCaptionParagraph(self.safe_area_x_offset, self.safe_area_y_offset, SccCaptionStyle.PopOn)

  def get_caption_to_process(self, time_code: SmpteTimeCode) -> Optional[SccCaptionParagraph]:
    """Returns the caption currently being processed"""
    if self.current_style in (SccCaptionStyle.PaintOn, SccCaptionStyle.RollUp):
      # If the Paint-On or Roll-Up style is activated, write directly on active caption
      if not self.has_active_caption():
        self.new_active_caption(time_code, self.current_style)
      return self.active_caption
    if self.current_style is SccCaptionStyle.PopOn:
      # For Pop-On style, write first on a buffered caption
      return self.buffered_caption

    LOGGER.warning("SCC caption style not defined")
    return None

  def has_active_caption(self) -> bool:
    """Returns whether captions are being displayed or not"""
    # return self.active_caption.get_begin() is not None
    return self.active_caption is not None

  def flip_buffered_to_active_captions(self, time_code: Optional[SmpteTimeCode] = None):
    """
    Flip the current buffered caption with the last active captions list,
    and push to model if an end time code is specified.
    """
    temporary_caption = None

    if self.has_active_caption():
      temporary_caption = self.active_caption

      if time_code is not None:
        # End of display of active captions
        self.push_active_caption_to_model(time_code)

    # Send the current buffered caption to the active captions list
    if not self.buffered_caption.get_id():
      self.count += 1
      self.buffered_caption.set_id("caption" + str(self.count))

    self.active_caption = self.buffered_caption
    self.new_buffered_caption()

    if temporary_caption is not None:
      self.buffered_caption = temporary_caption

  def push_active_caption_to_model(self, time_code: SmpteTimeCode, clear_active_caption: bool = True):
    """Sets end time to the last active caption, and pushes it into the data model"""
    if self.has_active_caption():
      self.active_cursor = self.active_caption.get_cursor()

      previous_caption = self.active_caption
      previous_caption.set_end(time_code)

      if clear_active_caption:
        self.active_caption = None

      if not previous_caption.is_empty():
        self.div.push_child(previous_caption.to_paragraph(self.div.get_doc()))

  def backspace(self, time_code: SmpteTimeCode):
    """Move the cursors in a column to the left"""
    caption = self.get_caption_to_process(time_code)
    caption.get_current_text().backspace()
    (row, indent) = caption.get_cursor()
    caption.set_cursor_at(row, max(indent - 1, 0))

  def paint_on_active_caption(self, time_code: SmpteTimeCode):
    """Initialize active caption for paint-on style"""
    active_style = SccCaptionStyle.PaintOn
    copied_lines = []
    cursor = self.active_cursor

    if self.has_active_caption():
      active_style = self.active_caption.get_caption_style()
      cursor = self.active_caption.get_cursor()

      # Copy buffered lines
      copied_lines = self.active_caption.copy_lines()

      # Push active to model if there is one
      self.push_active_caption_to_model(time_code)

    # Initialize new buffered caption
    self.new_active_caption(time_code, active_style)

    if len(copied_lines) > 0:
      # Set remaining lines to the new buffered caption
      self.active_caption.set_lines(copied_lines)

    self.active_caption.set_cursor_at(cursor[0], cursor[1])

  def process_preamble_address_code(self, pac: SccPreambleAddressCode, time_code: SmpteTimeCode):
    """Processes SCC Preamble Address Code it to the map to model"""

    pac_row = pac.get_row()
    pac_indent = pac.get_indent()

    if self.current_style is SccCaptionStyle.PaintOn:

      self.paint_on_active_caption(time_code)

      # Clear target row on Paint-On style
      target_row = self.active_caption.get_lines().get(pac_row)
      if target_row is not None:
        target_row.clear()

      self.active_caption.set_cursor_at(pac_row, pac_indent)

    elif self.current_style is SccCaptionStyle.RollUp:

      if not self.has_active_caption():
        # If there is no current active caption, initialize an empty new paragraph
        self.new_active_caption(time_code, SccCaptionStyle.RollUp)

      if self.active_caption.get_begin() is None:
        self.active_caption.set_begin(time_code)

      # Ignore PACs for rows 5-11, but get indent from PACs for rows 1-4 and 12-15. (Roll-Up)
      if pac_row in range(5, 12):
        self.active_caption.set_cursor_at(ROLL_UP_BASE_ROW)
        self.active_caption.new_caption_text()
        return

      # Force roll-up paragraph to belong to the same region
      self.active_caption.set_cursor_at(ROLL_UP_BASE_ROW, pac_indent)

      self.active_caption.new_caption_text()

    elif self.current_style is SccCaptionStyle.PopOn:
      # set cursor in paragraph and create line or text if necessary
      self.buffered_caption.set_cursor_at(pac_row, pac_indent)

    else:
      LOGGER.warning("SCC caption style not defined")

    self.current_color = pac.get_color()
    self.current_font_style = pac.get_font_style()
    self.current_text_decoration = pac.get_text_decoration()

    if self.has_active_caption():
      self.active_cursor = self.active_caption.get_cursor()

  def process_mid_row_code(self, mid_row_code: SccMidRowCode, time_code: SmpteTimeCode):
    """Processes SCC Mid-Row Code to map it to the model"""

    processed_caption = self.get_caption_to_process(time_code)

    color = mid_row_code.get_color()
    font_style = mid_row_code.get_font_style()
    text_decoration = mid_row_code.get_text_decoration()

    if self.previous_word_type is not SccMidRowCode:
      # In case of multiple mid-row codes, move right only after the first code

      # If there is already text on the current line
      if processed_caption is not None \
          and processed_caption.get_current_text() is not None \
          and not processed_caption.get_current_text().is_empty():

        # In case of paint-on replacing text
        if self.current_style is SccCaptionStyle.PaintOn \
            and processed_caption.get_current_line().get_cursor() < processed_caption.get_current_line().get_length():
          processed_caption.append_text(" ")

        else:
          if text_decoration is None:
            processed_caption.new_caption_text()
            processed_caption.append_text(" ")
          else:
            processed_caption.append_text(" ")
            processed_caption.new_caption_text()

      elif processed_caption is not None:
        processed_caption.append_text(" ")

      self.current_color = color
      self.current_font_style = font_style
      self.current_text_decoration = text_decoration

    else:
      if color is not None:
        self.current_color = color
      if font_style is not None:
        self.current_font_style = font_style
      if text_decoration is not None:
        self.current_text_decoration = text_decoration

      if processed_caption is not None:
        processed_caption.append_text(" ")
        processed_caption.new_caption_text()

    if processed_caption is not None \
        and processed_caption.get_caption_style() is SccCaptionStyle.PaintOn:
      processed_caption.get_current_text().set_begin(time_code)

  def process_attribute_code(self, attribute_code: SccAttributeCode, time_code: SmpteTimeCode):
    """Processes SCC Attribute Code to map it to the model"""

    processed_caption = self.get_caption_to_process(time_code)

    if processed_caption is None:
      LOGGER.warning("No current SCC caption nor content initialized")
      return

    if processed_caption.get_current_text().get_text():
      processed_caption.new_caption_text()

    if attribute_code.is_background():
      processed_caption.get_current_text().add_style_property(StyleProperties.BackgroundColor, attribute_code.get_color())
    else:
      processed_caption.get_current_text().add_style_property(StyleProperties.Color, attribute_code.get_color())

    processed_caption.get_current_text().add_style_property(StyleProperties.TextDecoration,
                                                            attribute_code.get_text_decoration())

  def process_control_code(self, control_code: SccControlCode, time_code: SmpteTimeCode):
    """Processes SCC Control Code to map it to the model"""

    if control_code is SccControlCode.RCL:
      # Start a new Pop-On caption
      self.current_style = SccCaptionStyle.PopOn

    elif control_code is SccControlCode.RDC:
      # Start a new Paint-On caption
      self.current_style = SccCaptionStyle.PaintOn

    elif control_code in (SccControlCode.RU2, SccControlCode.RU3, SccControlCode.RU4):
      # Start a new Roll-Up caption
      self.current_style = SccCaptionStyle.RollUp

      if control_code is SccControlCode.RU2:
        self.roll_up_depth = 2

      elif control_code is SccControlCode.RU3:
        self.roll_up_depth = 3

      elif control_code is SccControlCode.RU4:
        self.roll_up_depth = 4

      if not self.has_active_caption():
        # If there is currently no active caption, initialize an empty new paragraph
        self.new_active_caption(time_code, SccCaptionStyle.RollUp)

        self.active_caption.set_caption_style(SccCaptionStyle.RollUp)
        self.active_caption.set_cursor_at(ROLL_UP_BASE_ROW, 0)
        self.active_caption.new_caption_line()
        self.active_caption.new_caption_text()

        self.active_cursor = self.active_caption.get_cursor()

    elif control_code is SccControlCode.EOC:
      # Display caption (Pop-On)
      self.buffered_caption.set_begin(time_code)
      self.flip_buffered_to_active_captions(time_code)

      if self.has_active_caption():
        # Set text alignment
        if self.text_alignment == TextAlignment.AUTO:
          text_alignment = self.active_caption.guess_text_alignment()
        else:
          text_alignment = self.text_alignment.text_align

        # Apply text alignment
        self.active_caption.add_style_property(StyleProperties.TextAlign, text_alignment)

    elif control_code is SccControlCode.EDM:
      # Erase displayed captions
      if self.has_active_caption():
        self.push_active_caption_to_model(time_code)

    elif control_code is SccControlCode.ENM:
      # Erase buffered caption
      self.new_buffered_caption()

    elif control_code is SccControlCode.TO1:
      self.get_caption_to_process(time_code).indent_cursor(1)

    elif control_code is SccControlCode.TO2:
      self.get_caption_to_process(time_code).indent_cursor(2)

    elif control_code is SccControlCode.TO3:
      self.get_caption_to_process(time_code).indent_cursor(3)

    elif control_code is SccControlCode.CR:
      # Roll the displayed caption up one row (Roll-Up)

      if self.has_active_caption():
        if self.active_caption.get_caption_style() is not SccCaptionStyle.RollUp:
          LOGGER.warning("Cannot roll-up active %s-styled caption, erase it instead.", self.active_caption.get_caption_style().name)
          self.push_active_caption_to_model(time_code)
          return

        if self.active_caption.get_current_text().is_empty():
          self.count -= 1
          previous_lines = []
        else:
          # Push active caption to model (but don't erase it)
          self.push_active_caption_to_model(time_code, False)
          # Roll the active caption up
          self.active_caption.roll_up()
          # Get the remaining lines to initialize the following caption with the expected depth
          previous_lines = self.active_caption.get_last_caption_lines(self.roll_up_depth - 1)

        # Initialize the new caption with the previous lines
        self.new_active_caption(time_code, SccCaptionStyle.RollUp)
        self.active_caption.set_lines(previous_lines)

        self.active_caption.set_cursor_at(ROLL_UP_BASE_ROW)

    elif control_code is SccControlCode.DER:
      # Delete to End of Row (Paint-On)
      # The DER may be issued from any point on a row to delete all displayable characters, transparent
      # spaces, and mid-row codes from (and including) the current cell to the end of the row.
      # Not used in this implementation since this SCC reader does not map the text overlapping into
      # the model (i.e. a row is erased when a PAC is received, so before a new caption is written onto it).
      pass

    elif control_code is SccControlCode.BS:
      # Backspace
      # When a Backspace is received, the cursor moves to the left one column position erasing
      # the character or Mid-Row Code occupying that location, unless the cursor is in Column 1
      self.backspace(time_code)

  def process_text(self, word: str, time_code: SmpteTimeCode):
    """Processes SCC text words"""
    if self.current_style is SccCaptionStyle.PaintOn:
      if not self.has_active_caption():
        LOGGER.warning("Initialize active caption buffer to handle paint-on text at %s", time_code)
        self.paint_on_active_caption(time_code)

      if word.startswith(" "):

        if self.active_caption.get_caption_style() is not SccCaptionStyle.PaintOn:
          self.paint_on_active_caption(time_code)
          self.active_caption.append_text(word)

        else:
          self.active_caption.new_caption_text()
          self.active_caption.append_text(word)
          self.active_caption.get_current_text().set_begin(time_code)


      elif word.endswith(" "):
        self.active_caption.append_text(word)

        if self.active_caption.get_caption_style() is not SccCaptionStyle.PaintOn:
          self.paint_on_active_caption(time_code)
        else:
          self.active_caption.new_caption_text()
          self.active_caption.get_current_text().set_begin(time_code)

      else:
        if not self.has_active_caption():
          self.paint_on_active_caption(time_code)

        self.active_caption.append_text(word)

      self.active_caption.get_current_text().add_style_property(StyleProperties.Color, self.current_color)
      self.active_caption.get_current_text().add_style_property(StyleProperties.FontStyle, self.current_font_style)
      self.active_caption.get_current_text().add_style_property(StyleProperties.TextDecoration, self.current_text_decoration)

    elif self.current_style is SccCaptionStyle.RollUp:
      if not self.has_active_caption():
        LOGGER.warning("Initialize active caption buffer to handle roll-up text at %s", time_code)
        self.new_active_caption(time_code, self.current_style)

      self.active_caption.append_text(word)

      self.active_caption.get_current_text().add_style_property(StyleProperties.Color, self.current_color)
      self.active_caption.get_current_text().add_style_property(StyleProperties.FontStyle, self.current_font_style)
      self.active_caption.get_current_text().add_style_property(StyleProperties.TextDecoration, self.current_text_decoration)

    elif self.current_style is SccCaptionStyle.PopOn:
      self.buffered_caption.append_text(word)

      self.buffered_caption.get_current_text().add_style_property(StyleProperties.Color, self.current_color)
      self.buffered_caption.get_current_text().add_style_property(StyleProperties.FontStyle, self.current_font_style)
      self.buffered_caption.get_current_text().add_style_property(StyleProperties.TextDecoration, self.current_text_decoration)

    if self.has_active_caption():
      self.active_cursor = self.active_caption.get_cursor()

  def flush(self, time_code: Optional[SmpteTimeCode] = None):
    """Flushes the remaining current caption"""
    if self.has_active_caption():
      self.push_active_caption_to_model(time_code)

    # Remove the buffered caption
    self.new_buffered_caption()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC Extended characters"""

# pylint: disable=R0201,C0115,C0116,W0212

import unittest

from ttconv.scc.codes.extended_characters import SccExtendedCharacter


class SccExtendedCharactersTest(unittest.TestCase):

  def test_scc_spanish_extended_characters_unicode_values(self):
    extended_char = SccExtendedCharacter.find(0x1220)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_ACUTE, extended_char)
    self.assertEqual('\u00C1', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A20)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_ACUTE, extended_char)
    self.assertEqual('\u00C1', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1221)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_ACUTE, extended_char)
    self.assertEqual('\u00C9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A21)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_ACUTE, extended_char)
    self.assertEqual('\u00C9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1222)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_ACUTE, extended_char)
    self.assertEqual('\u00D3', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A22)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_ACUTE, extended_char)
    self.assertEqual('\u00D3', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1223)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_ACUTE, extended_char)
    self.assertEqual('\u00DA', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A23)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_ACUTE, extended_char)
    self.assertEqual('\u00DA', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1224)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00DC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A24)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00DC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1225)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_U_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00FC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A25)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_U_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00FC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1226)
    self.assertEqual(SccExtendedCharacter.LEFT_SINGLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u2018', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A26)
    self.assertEqual(SccExtendedCharacter.LEFT_SINGLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u2018', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1227)
    self.assertEqual(SccExtendedCharacter.INVERTED_EXCLAMATION_MARK, extended_char)
    self.assertEqual('\u00A1', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A27)
    self.assertEqual(SccExtendedCharacter.INVERTED_EXCLAMATION_MARK, extended_char)
    self.assertEqual('\u00A1', extended_char.get_unicode_value())

  def test_scc_miscellaneous_extended_characters_unicode_values(self):
    extended_char = SccExtendedCharacter.find(0x1228)
    self.assertEqual(SccExtendedCharacter.ASTERISK, extended_char)
    self.assertEqual('\u002A', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A28)
    self.assertEqual(SccExtendedCharacter.ASTERISK, extended_char)
    self.assertEqual('\u002A', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1229)
    self.assertEqual(SccExtendedCharacter.NEUTRAL_SINGLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u0027', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A29)
    self.assertEqual(SccExtendedCharacter.NEUTRAL_SINGLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u0027', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x122A)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_HORIZONTAL, extended_char)
    self.assertEqual('\u2501', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A2A)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_HORIZONTAL, extended_char)
    self.assertEqual('\u2501', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x122B)
    self.assertEqual(SccExtendedCharacter.COPYRIGHT_SIGN, extended_char)
    self.assertEqual('\u00A9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A2B)
    self.assertEqual(SccExtendedCharacter.COPYRIGHT_SIGN, extended_char)
    self.assertEqual('\u00A9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x122C)
    self.assertEqual(SccExtendedCharacter.SERVICE_MARK, extended_char)
    self.assertEqual('\u2120', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A2C)
    self.assertEqual(SccExtendedCharacter.SERVICE_MARK, extended_char)
    self.assertEqual('\u2120', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x122D)
    self.assertEqual(SccExtendedCharacter.BULLET, extended_char)
    self.assertEqual('\u2022', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A2D)
    self.assertEqual(SccExtendedCharacter.BULLET, extended_char)
    self.assertEqual('\u2022', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x122E)
    self.assertEqual(SccExtendedCharacter.LEFT_DOUBLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u201C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A2E)
    self.assertEqual(SccExtendedCharacter.LEFT_DOUBLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u201C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x122F)
    self.assertEqual(SccExtendedCharacter.RIGHT_DOUBLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u201D', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A2F)
    self.assertEqual(SccExtendedCharacter.RIGHT_DOUBLE_QUOTATION_MARK, extended_char)
    self.assertEqual('\u201D', extended_char.get_unicode_value())

  def test_scc_french_extended_characters_unicode_values(self):
    extended_char = SccExtendedCharacter.find(0x1230)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_GRAVE, extended_char)
    self.assertEqual('\u00C0', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A30)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_GRAVE, extended_char)
    self.assertEqual('\u00C0', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1231)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00C2', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A31)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00C2', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1232)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_C_WITH_CEDILLA, extended_char)
    self.assertEqual('\u00C7', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A32)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_C_WITH_CEDILLA, extended_char)
    self.assertEqual('\u00C7', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1233)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_GRAVE, extended_char)
    self.assertEqual('\u00C8', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A33)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_GRAVE, extended_char)
    self.assertEqual('\u00C8', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1234)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00CA', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A34)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00CA', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1235)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00CB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A35)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_E_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00CB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1236)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_E_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00EB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A36)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_E_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00EB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1237)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00CE', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A37)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00CE', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1238)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00CF', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A38)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00CF', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1239)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_I_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00EF', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A39)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_I_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00EF', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x123A)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00D4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A3A)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00D4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x123B)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_GRAVE, extended_char)
    self.assertEqual('\u00D9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A3B)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_GRAVE, extended_char)
    self.assertEqual('\u00D9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x123C)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_U_WITH_GRAVE, extended_char)
    self.assertEqual('\u00F9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A3C)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_U_WITH_GRAVE, extended_char)
    self.assertEqual('\u00F9', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x123D)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00DB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A3D)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_U_WITH_CIRCUMFLEX, extended_char)
    self.assertEqual('\u00DB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x123E)
    self.assertEqual(SccExtendedCharacter.LEFT_POINTING_GUILLEMET, extended_char)
    self.assertEqual('\u00AB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A3E)
    self.assertEqual(SccExtendedCharacter.LEFT_POINTING_GUILLEMET, extended_char)
    self.assertEqual('\u00AB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x123F)
    self.assertEqual(SccExtendedCharacter.RIGHT_POINTING_GUILLEMET, extended_char)
    self.assertEqual('\u00BB', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1A3F)
    self.assertEqual(SccExtendedCharacter.RIGHT_POINTING_GUILLEMET, extended_char)
    self.assertEqual('\u00BB', extended_char.get_unicode_value())

  def test_scc_portuguese_extended_characters_unicode_values(self):
    extended_char = SccExtendedCharacter.find(0x1320)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_TILDE, extended_char)
    self.assertEqual('\u00C3', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B20)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_TILDE, extended_char)
    self.assertEqual('\u00C3', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1321)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_A_WITH_TILDE, extended_char)
    self.assertEqual('\u00E3', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B21)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_A_WITH_TILDE, extended_char)
    self.assertEqual('\u00E3', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1322)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_ACUTE, extended_char)
    self.assertEqual('\u00CD', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B22)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_ACUTE, extended_char)
    self.assertEqual('\u00CD', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1323)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_GRAVE, extended_char)
    self.assertEqual('\u00CC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B23)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_I_WITH_GRAVE, extended_char)
    self.assertEqual('\u00CC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1324)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_I_WITH_GRAVE, extended_char)
    self.assertEqual('\u00EC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B24)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_I_WITH_GRAVE, extended_char)
    self.assertEqual('\u00EC', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1325)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_GRAVE, extended_char)
    self.assertEqual('\u00D2', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B25)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_GRAVE, extended_char)
    self.assertEqual('\u00D2', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1326)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_GRAVE, extended_char)
    self.assertEqual('\u00F2', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B26)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_GRAVE, extended_char)
    self.assertEqual('\u00F2', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1327)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_TILDE, extended_char)
    self.assertEqual('\u00D5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B27)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_TILDE, extended_char)
    self.assertEqual('\u00D5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1328)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_TILDE, extended_char)
    self.assertEqual('\u00F5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B28)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_TILDE, extended_char)
    self.assertEqual('\u00F5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1329)
    self.assertEqual(SccExtendedCharacter.BRACE_OPENING, extended_char)
    self.assertEqual('\u007B', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B29)
    self.assertEqual(SccExtendedCharacter.BRACE_OPENING, extended_char)
    self.assertEqual('\u007B', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x132A)
    self.assertEqual(SccExtendedCharacter.BRACE_CLOSING, extended_char)
    self.assertEqual('\u007D', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B2A)
    self.assertEqual(SccExtendedCharacter.BRACE_CLOSING, extended_char)
    self.assertEqual('\u007D', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x132B)
    self.assertEqual(SccExtendedCharacter.REVERSE_SOLIDUS, extended_char)
    self.assertEqual('\u005C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B2B)
    self.assertEqual(SccExtendedCharacter.REVERSE_SOLIDUS, extended_char)
    self.assertEqual('\u005C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x132C)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_TURNED_V, extended_char)
    self.assertEqual('\u028C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B2C)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_TURNED_V, extended_char)
    self.assertEqual('\u028C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x132D)
    self.assertEqual(SccExtendedCharacter.LOW_LINE, extended_char)
    self.assertEqual('\u005F', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B2D)
    self.assertEqual(SccExtendedCharacter.LOW_LINE, extended_char)
    self.assertEqual('\u005F', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x132E)
    self.assertEqual(SccExtendedCharacter.VERTICAL_LINE, extended_char)
    self.assertEqual('\u007C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B2E)
    self.assertEqual(SccExtendedCharacter.VERTICAL_LINE, extended_char)
    self.assertEqual('\u007C', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x132F)
    self.assertEqual(SccExtendedCharacter.TILDE, extended_char)
    self.assertEqual('\u007E', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B2F)
    self.assertEqual(SccExtendedCharacter.TILDE, extended_char)
    self.assertEqual('\u007E', extended_char.get_unicode_value())

  def test_scc_german_extended_characters_unicode_values(self):
    extended_char = SccExtendedCharacter.find(0x1330)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00C4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B30)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00C4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1331)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_A_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00E4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B31)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_A_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00E4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1332)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00D6', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B32)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00D6', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1333)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00F6', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B33)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_DIAERESIS, extended_char)
    self.assertEqual('\u00F6', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1334)
    self.assertEqual(SccExtendedCharacter.ESZETT, extended_char)
    self.assertEqual('\u00DF', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B34)
    self.assertEqual(SccExtendedCharacter.ESZETT, extended_char)
    self.assertEqual('\u00DF', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1335)
    self.assertEqual(SccExtendedCharacter.YEN_SIGN, extended_char)
    self.assertEqual('\u00A5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B35)
    self.assertEqual(SccExtendedCharacter.YEN_SIGN, extended_char)
    self.assertEqual('\u00A5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1336)
    self.assertEqual(SccExtendedCharacter.CURRENCY_SIGN, extended_char)
    self.assertEqual('\u00A4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B36)
    self.assertEqual(SccExtendedCharacter.CURRENCY_SIGN, extended_char)
    self.assertEqual('\u00A4', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1337)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_VERTICAL, extended_char)
    self.assertEqual('\u2503', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B37)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_VERTICAL, extended_char)
    self.assertEqual('\u2503', extended_char.get_unicode_value())

  def test_scc_danish_extended_characters_unicode_values(self):
    extended_char = SccExtendedCharacter.find(0x1338)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE, extended_char)
    self.assertEqual('\u00C5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B38)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE, extended_char)
    self.assertEqual('\u00C5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1339)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_A_WITH_RING_ABOVE, extended_char)
    self.assertEqual('\u00E5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B39)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_A_WITH_RING_ABOVE, extended_char)
    self.assertEqual('\u00E5', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x133A)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_STROKE, extended_char)
    self.assertEqual('\u00D8', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B3A)
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_O_WITH_STROKE, extended_char)
    self.assertEqual('\u00D8', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x133B)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_STROKE, extended_char)
    self.assertEqual('\u00F8', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B3B)
    self.assertEqual(SccExtendedCharacter.LATIN_SMALL_LETTER_O_WITH_STROKE, extended_char)
    self.assertEqual('\u00F8', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x133C)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_DOWN_AND_RIGHT, extended_char)
    self.assertEqual('\u250F', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B3C)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_DOWN_AND_RIGHT, extended_char)
    self.assertEqual('\u250F', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x133D)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_DOWN_AND_LEFT, extended_char)
    self.assertEqual('\u2513', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B3D)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_DOWN_AND_LEFT, extended_char)
    self.assertEqual('\u2513', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x133E)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_UP_AND_RIGHT, extended_char)
    self.assertEqual('\u2517', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B3E)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_UP_AND_RIGHT, extended_char)
    self.assertEqual('\u2517', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x133F)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_UP_AND_LEFT, extended_char)
    self.assertEqual('\u251B', extended_char.get_unicode_value())

    extended_char = SccExtendedCharacter.find(0x1B3F)
    self.assertEqual(SccExtendedCharacter.BOX_DRAWINGS_HEAVY_UP_AND_LEFT, extended_char)
    self.assertEqual('\u251B', extended_char.get_unicode_value())


if __name__ == '__main__':
  unittest.main()
